"""

    GreynirCorrect: Spelling and grammar correction for Icelandic

    Readability module

    Copyright © 2025 Miðeind ehf.

    This software is licensed under the MIT License:

        Permission is hereby granted, free of charge, to any person
        obtaining a copy of this software and associated documentation
        files (the "Software"), to deal in the Software without restriction,
        including without limitation the rights to use, copy, modify, merge,
        publish, distribute, sublicense, and/or sell copies of the Software,
        and to permit persons to whom the Software is furnished to do so,
        subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


    This module implements the Flesch reading ease score for Icelandic text.
    A high score indicates that the text is easy to read, while a low score
    indicates that the text is difficult to read.

"""

from __future__ import annotations
from typing import Dict, Iterable, List, Tuple

import math
import re
from enum import Enum

import tokenizer
from icegrams.ngrams import Ngrams
from islenska import Bin

diphtong_pattern = re.compile(r"(ei|ey|au)")
vowel_pattern = re.compile(r"[aeiouyáéíóúýöæ]")


class FleschKincaidFeedback(Enum):
    """The Flesch-Kincaid feedback of a text."""

    VERY_EASY_OR_INVALID = (120, "Mjög léttur eða ómarktækur texti.")
    VERY_EASY = (90, "Mjög léttur texti")
    EASY = (80, "Léttur texti")
    FAIRLY_EASY = (70, "Frekar léttur texti")
    STANDARD = (60, "Meðalléttur texti")
    FAIRLY_DIFFICULT = (50, "Svolítið þungur texti")
    DIFFICULT = (30, "Þungur texti")
    VERY_DIFFICULT = (0, "Mjög þungur texti")
    VERY_DIFFICULT_OR_INVALID = (-math.inf, "Mjög þungur eða ómarktækur texti.")

    def __ge__(self, other: FleschKincaidFeedback):
        if self.__class__ is other.__class__:
            return self.value >= other.value
        return NotImplemented

    def __gt__(self, other: FleschKincaidFeedback):
        if self.__class__ is other.__class__:
            return self.value > other.value
        return NotImplemented

    def __le__(self, other: FleschKincaidFeedback):
        if self.__class__ is other.__class__:
            return self.value <= other.value
        return NotImplemented

    def __lt__(self, other: FleschKincaidFeedback):
        if self.__class__ is other.__class__:
            return self.value < other.value
        return NotImplemented

    def __init__(self, score: float, feedback: str):
        self.score = score
        self.feedback = feedback

    def __str__(self):
        return self.feedback

    @staticmethod
    def from_score(score: float) -> FleschKincaidFeedback:
        """Provide feedback on the score. This is done by comparing the score to the
        Flesch-Kincaid reading ease score scale. The feedback has been slightly modified to fit Icelandic.
        Icelandic tends to score lower than English and thus the scale has been adjusted to reflect this.
        """
        for feedback in FleschKincaidFeedback:
            if score >= feedback.score:
                return feedback
        raise ValueError(f"Invalid score: {score}")


class FleschKincaidScorer:
    """Calculate the Flesch-Kincaid reading ease score of a given text."""

    @staticmethod
    def count_syllables_in_word(word: str) -> int:
        """Count the number of syllables in an Icelandic word, by counting the number of vowels in the word.
        This is done by first replacing diphtongs with a single character, then counting the number of vowels."""
        word = word.lower()
        # replace diphtongs with a single character
        word = diphtong_pattern.sub("a", word)
        return len(vowel_pattern.findall(word))

    @staticmethod
    def is_start_of_sentence(tok: tokenizer.Tok) -> bool:
        """Check if a token is the start of a sentence."""
        if tok.kind == tokenizer.TOK.S_BEGIN:
            return True
        return False

    @staticmethod
    def is_a_word(tok: tokenizer.Tok) -> bool:
        """Determine if a token is a word for the Flesch-Kincaid metric."""
        if tok.kind < tokenizer.TOK.META_BEGIN:
            # Punctuation marks are not counted as words
            if tok.kind > tokenizer.TOK.PUNCTUATION:
                return True
        return False

    @staticmethod
    def get_score(num_sentences: int, num_words: int, num_syllables: int) -> float:
        """Calculate the Flesch-Kincaid reading ease score after tracking a token stream.
        See https://en.wikipedia.org/wiki/Flesch%E2%80%93Kincaid_readability_tests."""
        # Magic numbers from the Flesch-Kincaid reading ease score formula - English
        # Icelandic tends to score lower than English and thus the scale has been adjusted to reflect this.
        # Original formula: 206.835 - (1.015 * (num_words / num_sentences)) - 84.6 * (num_syllables / num_words)
        score = 206.835 - (1.015 * (num_words / num_sentences)) - 70 * (num_syllables / num_words)
        return score

    @staticmethod
    def get_counts_from_stream(token_stream: Iterable[tokenizer.Tok]) -> Tuple[int, int, int]:
        num_words = 0
        num_sentences = 0
        num_syllables = 0
        for tok in token_stream:
            if FleschKincaidScorer.is_start_of_sentence(tok):
                num_sentences += 1
            elif FleschKincaidScorer.is_a_word(tok):
                num_words += 1
                if tok.kind == tokenizer.TOK.WORD:
                    num_syllables += FleschKincaidScorer.count_syllables_in_word(tok.txt)
                else:
                    # All tokens that are numbers, abbreviations, etc. are counted as one syllable, as an approximation
                    num_syllables += 1
            else:
                # All other tokens are ignored
                pass
        return num_sentences, num_words, num_syllables

    @staticmethod
    def get_score_from_stream(token_stream: Iterable[tokenizer.Tok]) -> float:
        """Calculate the Flesch-Kincaid reading ease score after tracking a token stream.
        See https://en.wikipedia.org/wiki/Flesch%E2%80%93Kincaid_readability_tests."""
        num_sentences, num_words, num_syllables = FleschKincaidScorer.get_counts_from_stream(token_stream)
        try:
            score = FleschKincaidScorer.get_score(num_sentences, num_words, num_syllables)
        except ZeroDivisionError:
            score = -1.0  # invalid score
        return score

    @staticmethod
    def get_score_from_text(text: str) -> float:
        """Calculate the Flesch-Kincaid reading ease score of a text.
        See https://en.wikipedia.org/wiki/Flesch%E2%80%93Kincaid_readability_tests."""
        stream = tokenizer.tokenize(text)
        return FleschKincaidScorer.get_score_from_stream(stream)

    @staticmethod
    def get_feedback(score: float) -> FleschKincaidFeedback:
        """Provide feedback on the score of the Result object. This is done by comparing the score to the
        Flesch-Kincaid reading ease score scale. The feedback has been slightly modified to fit Icelandic.
        Icelandic tends to score lower than English and thus the scale has been adjusted to reflect this.
        """
        return FleschKincaidFeedback.from_score(score)


class RareWordsFinder:
    """Find rare words in a text.

    Rare words are defined as words which have a probability lower than the low_prob_cutoff.
    The probability of a word is calculated by looking up the word in an n-gram model.
    The class is designed to be used with the tokenizer module and maintains an internal
    state which needs to be reset manually.
    """

    def __init__(self):
        self.bin = Bin()
        self.ng = Ngrams()

    def get_rare_words_from_stream(
        self, tok_stream: Iterable[tokenizer.Tok], max_words: int, low_prob_cutoff: float
    ) -> List[Tuple[str, float]]:
        """Tracks the probability of each word in a token stream.
        This is done by yielding the tokens in the token stream."""
        rare_words_dict: Dict[str, float] = {}
        for token in tok_stream:
            # Only consider words, not punctuation, numbers, etc.
            if token.kind == tokenizer.TOK.WORD:
                # unigram probability for the word
                prob = self.ng.prob(token.txt)
                if prob < low_prob_cutoff:
                    lemma_set = self.bin.lookup_lemmas_and_cats(token.txt)
                    lemma = lemma_set.pop()[0] if lemma_set else token.txt
                    # sometimes there are hyphens in the lemma
                    if "-" in lemma and "-" not in token.txt:
                        # remove the hyphen from the lemma
                        lemma = lemma.replace("-", "")
                    rare_words_dict[lemma] = prob
        rare_words = sorted(rare_words_dict.items(), key=lambda x: x[1], reverse=False)[:max_words]
        return rare_words

    def get_rare_words_from_text(
        self,
        text: str,
        max_words: int = 10,
        low_prob_cutoff: float = 0.00000005,
    ) -> List[Tuple[str, float]]:
        """Get the rare words from a text.
        This function is a convenience function which does not require the user to track the token stream.

        Rare words are defined as words which have a probability lower than the low_prob_cutoff.
        The probability of a word is calculated by looking up the word in the n-gram model.
        This function is a convenience function which does not require the user to track the token stream.
        """
        stream = tokenizer.tokenize(text)
        return self.get_rare_words_from_stream(stream, max_words, low_prob_cutoff)


if __name__ == "__main__":
    while True:
        # read the input from the user
        text = input("Sláðu inn texta: ").strip()
        max_words = 10
        low_prob_cutoff = 0.00000005

        flesch_score = FleschKincaidScorer.get_score_from_text(text)
        rare = RareWordsFinder()
        rare_words = rare.get_rare_words_from_text(text, max_words, low_prob_cutoff)

        print(f"Flesch-læsileikastig: {flesch_score:.2f}")
        print(f"Flesch-umsögn: {FleschKincaidScorer.get_feedback(flesch_score)}")
        print("Sjaldgæfustu orð í textanum:")
        for word, _ in rare_words:
            print(f"\t{word}")
        print()



# type: ignore
"""

    test_patterns.py

    Tests for GreynirCorrect module

    Copyright © 2025 by Miðeind ehf.

    This software is licensed under the MIT License:

        Permission is hereby granted, free of charge, to any person
        obtaining a copy of this software and associated documentation
        files (the "Software"), to deal in the Software without restriction,
        including without limitation the rights to use, copy, modify, merge,
        publish, distribute, sublicense, and/or sell copies of the Software,
        and to permit persons to whom the Software is furnished to do so,
        subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


    This module tests the grammatical pattern annotation functionality
    of GreynirCorrect.

"""

import pytest

import reynir_correct

from .utils import check_sentence


@pytest.fixture(scope="module")
def api() -> reynir_correct.GreynirCorrectAPI:
    """Provide a module-scoped GreynirCorrect instance as a test fixture"""
    r = reynir_correct.GreynirCorrectAPI.from_options()
    yield r


def test_verb_af(api):
    s = "Ráðherrann dáðist af hugrekki stjórnarandstöðunnar."
    check_sentence(api, s, [(2, 2, "P_WRONG_PREP_AF")])

    s = "Mig langaði að leita af bílnum, en dáðist svo af hugrekki lögreglukonunnar " "að ég gerði það ekki."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF"), (10, 10, "P_WRONG_PREP_AF")])

    s = "Við höfum leitað í allan dag af kettinum, en fundum hann ekki."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])

    s = "Allan daginn höfum við leitað af kettinum."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AF")])

    s = "Páll brosti af töktunum í Gunnu."
    check_sentence(api, s, [(2, 2, "P_WRONG_PREP_AF")], ignore_warnings=True)

    s = "Ég var leitandi af kettinum í allan dag."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Ég vildi leita af mér allan grun."
    check_sentence(api, s, [])

    s = "Hver leitar af skrifstofuhúsnæði?"
    check_sentence(api, s, [(2, 2, "P_WRONG_PREP_AF")])

    s = "Hann dáist endalaust af þeim."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Hann hefur lengi dáðst af þeim."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])

    s = "Jón gerir grín af því."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Þetta er mesta vitleysa sem ég hef gert grín af."
    check_sentence(api, s, [(9, 9, "P_WRONG_PREP_AF")])

    s = "Jón kann það ekki utan af."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AF")])

    s = "Jón leggur hann ekki af velli."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])

    s = "Jón hefur ekki lagt hann af velli."
    # TODO "af velli" ends up under "hann"
    # check_sentence(rc, s, [(1, 6, "P_WRONG_PREP_AF")])

    s = "Jón leiðir líkur af því."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Jón leiðir ekki líkur af því."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])

    s = "Jón leiðir rök af því."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Jón leitar af því."
    check_sentence(api, s, [(2, 2, "P_WRONG_PREP_AF")])

    s = "Tíminn markar upphaf af því."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Tíminn markar ekki upphaf af því."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])

    s = "Það markar upphafið af því."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Það markar ekki upphafið af því."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])

    s = "Það hefur ekki markað upphafið af því."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AF")])

    s = "Jón spyr af því."
    # TODO not picked up
    # check_sentence(rc, s, [(1, 2, "P_WRONG_PREP_AF")])

    s = "Það sem Jón spurði ekki af var óljóst."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AF")])

    s = "Jón stuðlar af því."
    # TODO not picked up
    # check_sentence(rc, s, [(2, 2, "P_WRONG_PREP_AF")])

    s = "Honum varð af ósk sinni."
    check_sentence(api, s, [(2, 2, "P_WRONG_PREP_AF")])

    s = "Honum hafði orðið af ósk sinni."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Honum varð ekki af ósk sinni."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])

    s = "Hann varð ekki uppvís af því."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])

    s = "Jón varð vitni af þessu."
    check_sentence(api, s, [(1, 3, "P_afað"), (3, 3, "S005")])

    s = "Hún er ólétt af sínu þriðja barni."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])
    #    s = "Það kom henni á óvart að hún væri ólétt af strák." # Annotation variable depending on parse
    #    check_sentence(rc, s, [(8, 9, "P_WRONG_PREP_AF")])
    s = "Að öllu leyti eru öll ólétt af stelpum."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])


def test_noun_af(api):
    s = "Hann gerði þetta af beiðni hennar."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])
    s = "Af beiðni hennar gerði hann þetta."
    check_sentence(api, s, [(0, 0, "P_WRONG_PREP_AF")])
    s = "Það var gert af þeirri fyrirmynd."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])
    s = "Þau gera þetta af heiðnum sið."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])
    s = "Ég baka köku af því tilefni."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])
    s = "Þau veittu mér aðgang af kerfinu."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])
    s = "Aðgangur af kerfinu var veittur."
    check_sentence(api, s, [(1, 1, "P_WRONG_PREP_AF")])
    s = "Drög af verkefninu eru tilbúin."
    check_sentence(api, s, [(1, 1, "P_WRONG_PREP_AF")])
    #    s = "Þau kláruðu drög af verkefninu."
    #    check_sentence(rc, s, [(2, 3, "P_WRONG_PREP_AF")])
    s = "Grunnur af verkefninu er tilbúinn."
    check_sentence(api, s, [(1, 1, "P_WRONG_PREP_AF")])
    #    s = "Hann lagði ekki grunninn af verkefninu."
    #    check_sentence(rc, s, [(3, 4, "P_WRONG_PREP_AF")])
    #    s = "Þau gerðu leit af dótinu."
    #    check_sentence(rc, s, [(2, 3, "P_WRONG_PREP_AF")])
    s = "Leit af dótinu hefur ekki skilað árangri."
    check_sentence(api, s, [(1, 1, "P_WRONG_PREP_AF")])
    s = "Þetta er lykillinn af velgengni."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AF")])
    s = "Hann gaf mér uppskriftina af réttinum."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AF")])


def test_verb_að(api):
    s = "Ég er ekki hluti að heildinni."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Við höfum öll verið hluti að heildinni."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ")])
    s = "Vissulega er hægt að vera hluti að heildinni."
    check_sentence(api, s, [(6, 6, "P_WRONG_PREP_AÐ")])
    #    s = "Þeir sögðu að ég hefði verið hluti að heildinni."   # Annotation variable depending on parsing
    #    check_sentence(rc, s, [(6, 8, "P_WRONG_PREP_AÐ")])
    s = "Að öllu óbreyttu er hann hluti að heildinni."
    check_sentence(api, s, [(6, 6, "P_WRONG_PREP_AÐ")])
    s = "Ég vildi vera hluti að heildinni að mestu leyti."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Þar að leiðandi virkar þetta."
    check_sentence(api, s, [(0, 2, "P_aðaf"), (1, 1, "S005")])
    s = "Þar að leiðandi virkar þetta að mestu leyti."
    check_sentence(api, s, [(0, 2, "P_aðaf"), (1, 1, "S005")])
    s = "Þetta virkar þar að leiðandi."
    check_sentence(api, s, [(2, 4, "P_aðaf"), (3, 3, "S005")])
    s = "Ég hef ekki áhyggjur að honum."
    # check_sentence(rc, s, [(2, 5, "P_WRONG_PREP_AÐ")])
    s = "Ég hef áhyggjur að því að honum líði illa."
    # check_sentence(rc, s, [(2, 8, "P_WRONG_PREP_AÐ")])
    s = "Ég lagði ekki mikið að mörkum."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Ég hafði lagt mikið að mörkum."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Sama hvað ég gerði lagði ég mikið að mörkum."
    check_sentence(api, s, [(7, 7, "P_WRONG_PREP_AÐ")])
    s = "Að hans mati lagði hann mikið að mörkum."
    check_sentence(api, s, [(6, 6, "P_WRONG_PREP_AÐ")])
    s = "Ég heillast að þannig fólki."
    check_sentence(api, s, [(2, 2, "P_WRONG_PREP_AÐ")])
    s = "Það að heillast að þannig fólki er algengt."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AÐ")])
    s = "Ég lét gott að mér leiða."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AÐ")])
    s = "Að mínu mati lét ég ekki gott að mér leiða."
    check_sentence(api, s, [(7, 7, "P_WRONG_PREP_AÐ")])
    s = "Hún á heiðurinn að þessu."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AÐ")])
    s = "Hún hafði ekki átt heiðurinn að þessu en fékk heiðurinn að þessu."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ"), (10, 10, "P_WRONG_PREP_AÐ")])
    s = "Hún hlaut heiðurinn að þessu."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AÐ")])
    s = "Að endingu hljótum við öll heiðurinn að því."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ")])
    s = "Hún á heilan helling að börnum."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Hún á marga að."
    check_sentence(api, s, [])
    s = "Að mínu mati eigum við mikið að mat."
    check_sentence(api, s, [(6, 6, "P_WRONG_PREP_AÐ")])
    s = "Hún á ekki aðild að málinu."
    check_sentence(api, s, [])
    s = "Hún hefur ekki haft gagn að þessu."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ")])
    s = "Að mínu mati hef ég ekki gagn að þessu."
    # check_sentence(rc, s, [(3, 8, "P_WRONG_PREP_AÐ")])
    s = "Þetta hafði ekki komið að sjálfu sér."
    check_sentence(api, s, [(4, 6, "P_aðaf")])
    s = "Fréttir bárust seint að slysinu."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AÐ")])
    s = "Að endingu berast fréttir að slysinu."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AÐ")])
    s = "Þetta er afgreitt mál að minni hálfu."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Hætta hefur aldrei stafað að þessu."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    #    s = "Að mínu mati stafar ekki hætta að þessu."  # Annotation variable depending on parse
    #    check_sentence(rc, s, [(3, 7, "P_WRONG_PREP_AÐ")])
    s = "Hann hefur ekki heyrt að lausa starfinu."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Að endingu heyrði ég að starfinu."
    check_sentence(api, s, [(3, 3, "P_WRONG_PREP_AÐ")])
    s = "Ég hef aldrei haft gaman að henni."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ")])
    #    s = "Ég sagði að ég hefði gaman að henni."  # Annotation variable depending on parse
    #    check_sentence(rc, s, [(5, 7, "P_WRONG_PREP_AÐ")])
    s = "Þau voru sérstaklega valin að stjórninni."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    #    s = "Þú ert valinn að guði að okkar mati."  # Annotation variable depending on parse
    #    check_sentence(rc, s, [(1, 7, "P_WRONG_PREP_AÐ")])
    s = "Það er til mjög lítið að mjólk."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ")])
    s = "Ekki er mikið til að mjólk."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Að öllu leyti er til fullt að mjólk."
    check_sentence(api, s, [(4, 4, "P_WRONG_PREP_AÐ")])
    s = "Ég hef ekki unnið verkefni að þessu tagi."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ")])
    s = "Verkefni að þessum toga eru erfið."
    check_sentence(api, s, [(1, 1, "P_WRONG_PREP_AÐ")])
    s = "Að mínu mati gerði ég þetta að krafti."
    check_sentence(api, s, [(6, 6, "P_WRONG_PREP_AÐ")])
    s = "Hann gerði það að sjálfsdáðum."
    check_sentence(api, s, [(3, 4, "P_aðaf")])
    s = "Að sjálfsögðu gerði ég þetta að sjálfsdáðum."
    check_sentence(api, s, [(4, 5, "P_aðaf")])
    s = "Hún hefur ekki gert þetta að miklum krafti."
    check_sentence(api, s, [(5, 5, "P_WRONG_PREP_AÐ")])


def test_placename_pp(api):
    s = "Ég hef búið á Hafnarfirði alla mína tíð en flyt nú í Akureyri."
    check_sentence(api, s, [(3, 4, "P_WRONG_PLACE_PP"), (9, 10, "P_WRONG_PLACE_PP")])
    s = "Ég hef veitt í Vopnafirði undanfarin ár en búið á Vopnafirði."
    check_sentence(api, s, [])
    s = "Það eru mörg náttúruvætti á Reykjanesi en ekki í Húsavík."
    check_sentence(api, s, [(8, 9, "P_WRONG_PLACE_PP")])


def test_verb_líst(api):
    s = "Jóni veiðimanni lýst ekki á þetta mál."
    # check_sentence(rc, s, [(2, 2, "P_WRONG_OP_FORM")])
    check_sentence(api, s, [(2, 2, "Ý4Í")])
    s = "Eins og fram hefur komið lýst mér vel á þetta."
    check_sentence(api, s, [(5, 5, "Ý4Í")])
    s = "Jón hefur lýst sinni afstöðu til málsins."
    # TODO: 'Corrected' at token-level, in the works to fix at sentence-level
    # check_sentence(rc, s, [])
    s = "Þegar leið á kvöldið var gangstéttin lýst með ljósum."
    # check_sentence(rc, s, [], ignore_warnings=True)
    # TODO: The following gets no annotation:
    "Ég verð að segja að mér lýst ekkert á þetta."
    check_sentence(api, s, [(6, 6, "Ý4Í")])


def test_dir_loc(api):
    s = "Börnin voru út á túni allan daginn."
    check_sentence(api, s, [(2, 4, "P_DIR_LOC")])
    s = "Börnin voru útá túni allan daginn."
    check_sentence(api, s, [(2, 3, "P_DIR_LOC"), (2, 2, "W001/w")])
    #    s = "Út í heimi er þetta öðruvísi."
    #    check_sentence(rc, s, [(0, 2, "P_DIR_LOC")])
    #    s = "Útí heimi er þetta öðruvísi."
    #    check_sentence(rc, s, [(0, 1, "P_DIR_LOC")])
    s = "Börnin voru inn á vellinum allan daginn."
    check_sentence(api, s, [(2, 4, "P_DIR_LOC")])
    s = "Börnin voru inná vellinum allan daginn."
    check_sentence(api, s, [(2, 3, "P_DIR_LOC"), (2, 2, "W001/w")])
    #    s = "Hann var oft upp á hestinum."
    #    check_sentence(rc, s, [(3, 5, "P_DIR_LOC")])
    s = "Málið liggur í augum upp."
    check_sentence(api, s, [(2, 4, "P_DIR_LOC")])
    #    s = "Þau eru alltaf uppí bústað."
    #    check_sentence(rc, s, [(1, 4, "P_DIR_LOC"), (3, 3, "W001/w")])
    # Span is either 1,4 or 3,4 (depending on parsing), but always corrected.
    s = "Hún var niður í bæ í gær."
    check_sentence(api, s, [(1, 5, "P_DIR_LOC")])
    #    s = "Ég varð mér út um smá mat."
    #    check_sentence(rc, s, [(3, 6, "P_DIR_LOC")])
    #    s = "Þegar upp er staðið erum við öll eins."
    #    check_sentence(rc, s, [(1, 3, "P_DIR_LOC")])
    #    s = "Út í heimi er þetta öðruvísi."
    #    check_sentence(rc, s, [(0, 2, "P_DIR_LOC")])
    s = "Börnin safnast saman inn í búð."
    check_sentence(api, s, [(3, 5, "P_DIR_LOC")])
    s = "Ég keypti þetta út í búð."
    check_sentence(api, s, [(3, 5, "P_DIR_LOC")])
    s = "Illgresið er út um allt."
    check_sentence(api, s, [(2, 4, "P_DIR_LOC")])
    #    s = "Hann læsti sig inn í gær."
    #    check_sentence(rc, s, [(1, 4, "P_DIR_LOC")])
    s = "Hún gaf það upp í fréttum."
    check_sentence(api, s, [])
    s = "Ég ólst upp í Breiðholtinu."
    check_sentence(api, s, [])



# type: ignore
"""

    test_serializers.py

    Tests for JSON serialization of sentences

    Copyright © 2025 by Miðeind ehf.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""

import json

import reynir_correct as rc


def test_serializers():
    sents = [
        "Ég fór niðrá bryggjuna með með Reyni Vilhjálmssyni í gær.",
        "Það var 17. júní árið 2020 í frakklandi.",
        "Við sáum tvo seli og öruglega fleiri en 100 máva.",
        "Klukkan var orðinn tólf þegar við fórum heim.",
        "Bíllinn kostaði €30.000 en ég greyddi 25500 USD fyrir hann.",
        "morguninn eftir vakknaði ég kl. 07:30.",
        "Ég var firstur á fætur en þuríður Hálfdánardóttir var numer 2.",
    ]

    # Global settings object for the tests
    api = rc.GreynirCorrectAPI.from_options()
    gc = api.gc

    job = gc.submit(sents, parse=True)
    for pg in job.paragraphs():
        for sent in pg:
            assert sent.tree is not None

            json_str = gc.dumps_single(sent, indent=2)
            new = gc.loads_single(json_str)

            assert new.tree is not None

            assert sent.tokens == new.tokens
            assert sent.terminals == new.terminals
            assert sent.tree.flat_with_all_variants == new.tree.flat_with_all_variants

            cls = gc.__class__
            assert json.loads(sent.dumps(cls, indent=2)) == json.loads(new.dumps(cls, indent=2))


if __name__ == "__main__":
    # When invoked as a main module, do a verbose test
    test_serializers()



from typing import Tuple

import tokenizer

from reynir_correct import readability

hard_text1 = "Í Ritgerð í átt að nýrri kenningu um sjón (An essay towards a new theory of vision)2 sem kom út 1709 tekur Berkeley sérstaklega fram að þær lögunarhugmyndir sem skynjaðar eru með sjóninni séu aðrar en þær sem skynjaðar eru með snertiskyninu og að þessi tvö skynfæri eigi engar sameiginlegar hugmyndir; við sjáum ekki það sem við snertum, eða öfugt. Berkeley telur, líkt og Locke, að sjónskynið eitt og óstutt gefi okkur ekki þrívíddarhugmyndir þar sem dýptar- og fjarlægðarskyn komi ekki til nema með reynslu. Þeir sem hafa reynslu af því að sjá og snerta hluti hafa lært að tilteknar sjónrænar hugmyndir og tilteknar snertihugmyndir eiga það til að fylgjast að og þess vegna tengjum við þær saman. En maður sem skyndilega fær sjónina hefur ekki þessa reynslu til að byggja á og dregur því enga ályktun um snertihugmyndir á grundvelli þess sem fyrir augu hans ber. Hið neikvæða svar Berkeleys við spurningu Molyneux sem hann ræðir stuttlega síðar í þessu riti sínu um sjónina liggur því beint við."

hard_text2 = "Ábyrgðaraðila staðarins er skylt að tilkynna heilbrigðisnefnd um viðburði þar sem búast má við að hljóðstig verði hærra en tilgreint er í 1. mgr. með hæfilegum fyrirvara og ber hann kostnað við eftirlit á tónleikunum þ.m.t. hljóðmælingum skv. gjaldskrá. Heilbrigðisnefnd getur beint þeim tilmælum til ábyrgðaraðila staðarins að hann bjóði gestum eyrnatappa og hengi upp sérstök viðvörunarskilti um hátt hljóðstig á áberandi hátt, setji aldurstakmörk fyrir gesti, geri grein fyrir staðsetningu hátalara og öðru sem heilbrigðisnefnd telur þurfa til að koma í veg fyrir heilsuspillandi hávaða. Brot gegn ákvæðum 6.-10. gr. reglugerðar þessarar varða sektum hvort sem þau eru framin af ásetningi eða stórfelldu gáleysi. Sé um stórfelld eða ítrekuð ásetningsbrot að ræða skulu þau að auki varða fangelsi allt að fjórum árum. Sektir má ákvarða lögaðila þó að sök verði ekki sönnuð á fyrirsvarsmenn eða starfsmenn hans eða aðra þá einstaklinga sem í þágu hans starfa, enda hafi brotið orðið eða getað orðið til hagsbóta fyrir lögaðilann. Þó skal lögaðili ekki sæta refsingu ef um óhapp er að ræða. Einnig má, með sama skilorði, gera lögaðila sekt ef fyrirsvarsmenn eða starfsmenn hans eða aðrir einstaklingar sem í þágu hans starfa gerast sekir um brot."

medium_hard_text = "Íslandsbanki lítur misferli mjög alvarlegum augum. Við hvetjum þig til að láta vita ef þú hefur vitneskju eða grun um hugsanlegt misferli sem tengist starfsemi bankans á einhvern hátt. Þar með aðstoðar þú við að upplýsa um brot sem valdið geta viðskiptavinum, almenningi, bankanum og atvinnulífi miklu tjóni. Tilkynning um misferli þarf að byggja á rökstuddum grun, sem þarf þó ekki að vera hafinn yfir allan vafa. Móttaka misferlistilkynninga fer í gegnum kerfið WhistleB sem er sérstaklega aðgreint frá öðrum upplýsingakerfum bankans og vistað utan hans. Kerfið er sérstaklega hannað með tilliti til verndunar tilkynnanda og þeirra tilkynninga sem þangað berast, en veitir jafnframt möguleika á áframhaldandi samskiptum við þá sem kjósa að njóta nafnleyndar. Kerfið er mjög einfalt í notkun og leiðir tilkynnanda áfram. Til að tryggja nafnleynd er starfsmönnum bent á að skrá ekki tilkynningar í gegnum tölvubúnað sem tengist netkerfi bankans. Einungis misferlisteymi innri endurskoðunar bankans mun hafa aðgang að skráðum málum."

medium_text1 = "Verðbólgan er komin í 10,2%. Það er allt of mikið og hefur heilmikil áhrif á okkur öll. Í Korni Greiningar Íslandsbanka er greint frá ástæðum þessarar miklu verðbólgu og spáð fyrir um þróunina á næstunni. Slíkt efni má nálgast frá okkur reglulega með skráningu á póstlistann okkar. Hér langar okkur þó að stíga eitt skref til baka og rýna í þessar verðbólgutölur með öðrum hætti. Lítum á 5 áhugaverðar staðreyndir um þessa miklu verðbólgu í dag. Hvað hefur hækkað mest? Ef þér finnst allt vera að hækka ertu með puttann nokkuð vel á verðbólgupúlsinum. Verðhækkanirnar sem búa til þessa miklu verðbólgu koma úr ýmsum áttum og má sem dæmi nefna að matvörur hafa margar hverjar hækkað mikið í verði undanfarið ár. Þá hefur flutningsverð og kostnaður við ferðalög rokið upp sem og húsnæðisverð, þó svo húsnæðismarkaðurinn sé að kólna ansi hratt þessa dagana. Ef við lítum á nokkur dæmi um hækkanir vekur kannski athygli að loksins virðist gamli góði kálböggullinn vera orðinn lúxusvara. Eða þangað virðist hann í það minnsta stefna því kál hefur hækkað um heilt 31% í verði frá því á sama tíma í fyrra og munar um minna. Það er orðið talsvert dýrara að ferðast (og að verja tíma erlendis sömuleiðis) og loks þarf þessi væna flís af feitum sauð sem ég sá fyrir mér um helgina að vera aðeins minni því lambakjötið okkar er 19% dýrara en í fyrra."

medium_text2 = "Tónlistarmaðurinn Daði Freyr sendi frá sér sína fyrstu plötu í fullri lengd í ágúst. Fyrsta lag hans kom út fyrir sex árum og síðan hefur fylgt hellingur af lögum og þó nokkrar þröngskífur og alls konar stemmning eins og alþjóð veit. Það er kannski hálfskrítið að tónlistarmaður eins og Daði Freyr sé að senda frá sér sína fyrstu plötu í lok ágúst árið 2023 en það þýðir nú ekki að hann sé búinn að vera latur. Eins og fólk veit hefur hann verið að taka þátt í Söngvakeppni evrópskra sjónvarpsstöðva, skemmta í sömu keppni og svo var covid og alls konar dæmi sem tekur tíma. Nóg um það, nýja platan er tekin upp á heimili kappans í Berlín og smíði hennar hófst fyrir tveimur árum. Daði segir plötuna á persónulegum nótum, meðganga og fæðing auk heimsfaraldurs hafi litað sköpunina ásamt þessum venjulegu áhrifavöldum eins og tónlist sem hann hefur verið að hlusta á, vinum og stórfjölskyldu. Plata vikunnar að þessu sinni er I Made An Album, fyrsta plata Daða Freys, og er aðgengileg í heild sinni ásamt kynningum hans í spilara RÚV."

easy_text1 = "Þegar ég fæddist fékk ég stimpilinn „stelpa“. Ég valdi ekki þennan stimpil, enda hafði ég hvorki vit né skoðun á málinu þegar ég hlaut þennan titil. Ég ólst upp með þennan stimpil, en það er ýmislegt sem fylgir honum. Ég fékk alveg að vera ég sjálfur þannig, en ég vissi samt að ég ætti að haga mér og klæða mig á ákveðinn hátt. Það að eiga að vera stelpa truflaði mig ekki, ég var töffari („tomboy“) og gerði að mestu bara það sem ég vildi. Ég óskaði þess samt alltaf að hafa fæðst sem strákur, alveg frá því að ég var krakki og langt fram eftir aldri, fannst það passa mér betur, vera meira spennandi. Þegar ég varð unglingur fór ég oftar að reyna að passa í kassann, svo að segja. Ég reyndi m.a. að vera pæja, en það entist aldrei lengi, ég varð mikill femínisti og það hjálpaði mér að halda áfram að gera það sem ég vildi. Ég fór ekki að pæla í kyninu mínu af alvöru fyrr en ég var orðinn 19 ára. Ég horfði á myndband, og fattaði allt í einu að ég gæti verið trans, að ég mætti vera strákur, að það væri hægt. Það var eins og það hefði kviknað ljós í höfðinu á mér. Ég hugsaði um lítið annað næstu mánuðina. Í hvert skipti sem ég þurfti að kynja mig í hugsunum mínum, sem er rosalega oft ef þið hugsið út í það, fór ég að hugsa um kyn mitt: „Ég er svöng … svangur? … svangt? Hvað er ég?“ Ég sagði kærustunni minni eiginlega strax hvað ég var að hugsa og hún bauðst til að nota karlkyn um mig þegar við værum ein. Eftir eina viku var jafn skrítið að hún notaði karlkyn eins og að allir aðrir notuðu kvenkyn, eins og þau höfðu gert í meira en 19 ár."

easy_text2 = "Ernir eru stórir ránfuglar. Það er ein tegund af örnum á Íslandi, haförn. Haförn er næst-stærsta tegund af fuglum á Íslandi, á eftir álft eða svani. En örnum hefur ekki gengið vel á Íslandi. Á síðustu öld var hætta á að þeir myndu deyja út. Þá voru bara um 20 pör af örnum á Íslandi, eða um 40 ernir. Þá var Fuglaverndarfélagið stofnað til að hjálpa örnum. Félagið lét banna að eitra fyrir refum. Margir ernir dóu nefnilega út af refaeitri. Eftir það fór örnum að ganga betur. Þeim fjölgaði hægt og rólega. Núna eru um 90 pör af örnum á Íslandi, eða um 180 ernir. Fuglafræðingur segir að örnum hafi gengið vel að eignast unga á þessu ári. Um 45 pör eru núna með unga. Það hefur sjaldan gengið betur. Ef þetta heldur áfram munu ernir aftur dreifa sér um Ísland. Þá fara ernir aftur að sjást á fleiri stöðum. Núna eru ernir aðallega á Vesturlandi. Einu sinni voru þeir um allt land, nema á hálendinu."

numbers = "123456677, 2434,5, 5 23 233 , ! ???' % #$43234654 1 23 45667 7, 2434,5, 5 23 233 , ! ?  ?' % #$ 432 34 654 12345 6677, 243 4,5, 5 23 233 , ! ???' % #$43 23 4654 123456677, 2434,5, 5 23 233 , ! ???' % #$43234654 123456677. 2434,5, 5 23 233 , ! ??.?' % #$43.2 3465 4 12 345 6677, 2434,5, 5 23 23 3 , ! ???' % #$43 234 654 ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? 12345.6677, 2.434,5, 5 23 233 ., ! ???' % #$43234654 1 23 45667 7, 2434,5, 5 23 233 , ! ?  ?'. % #$ 432 34 654 12345 6677, 243 4,5, 5 23 233 , ! ???' % #$43 23 4654 123456677, 2434,5, 5 23 233 , ! ???' % #$43234.654 123456677, 2434,5, 5 23 233 , ! ???' % #$43.2 3465 4 12 345 6677, .2434,5, 5 23 23 3 , ! ???' % #$43 234 654 ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?"

single_syllables = "Ég á nú að fá ís í búð í dag. Þú ert frá þér af sorg af því þú drakkst ei nóg af bjór."

many_syllables = "Ævinlega þegar Elísabet vaknaði lagaði hún rótsterkan kaffibolla handa sjálfri sér og svefndrukknum eiginmanninum, honum Engilbert. Aukinheldur íklæddist hún föðurlandi og fjólublárri herðaslá."

many_numbers_and_such = "Hafðu samband við ráðgjafaver Íslandsbanka í síma 440 4000, sem er opið í síma frá klukkan 9-16 og netspjalli frá 9-17 alla virka daga. Utan opnunartíma er hægt að hafa samband við neyðarþjónustu Íslandsbanka í síma 440 4000 eða á islandsbanki.is. Árið 1944 þann 17. júní fengu Íslendingar sjálfstæði frá Dönum. 37°C er algengur líkamshiti. Fólk í Grænumýri 11 lenti í 4. lekanum í gær, 30. ágúst."


no_text = ""
# text and expected range:
text_dict = {
    easy_text1: (70, 90),
    easy_text2: (70, 100),
    medium_text2: (50, 70),
    medium_text1: (50, 70),
    medium_hard_text: (40, 60),
    hard_text1: (20, 50),
    hard_text2: (20, 50),
}

not_real_texts = {
    numbers: (100, 150),  # no text
    single_syllables: (90, 130),  # very improbable text
    many_syllables: (-10, 10),  # very improbable text
    many_numbers_and_such: (50, 100),
}


def get_sentence_word_syllable_counts(text: str) -> Tuple[int, int, int]:
    """Get the number of sentences, words and syllables in a text."""
    stream = tokenizer.tokenize(text)
    return readability.FleschKincaidScorer.get_counts_from_stream(stream)


def test_count_syllables():
    words_and_syllables = [
        ("hús", 1),
        ("húsið", 2),
        ("húsin", 2),
        ("húsinu", 3),
        ("ae_iouyáéíóúýöæ", 14),  # vowels
        ("eieyau", 3),  # diphtongs
        ("bcdfghjklmnpqrstvwxzðþ", 0),  # consonants
        ("áin þessi", 4),  # spaces
        ("", 0),
        (" ", 0),
        ("1999", 0),
    ]
    for word, num_syllables in words_and_syllables:
        found_syllables = readability.FleschKincaidScorer.count_syllables_in_word(word)
        assert (
            found_syllables == num_syllables
        ), f"Expected {num_syllables} syllables in {word} but got {found_syllables}"


def test_count_sentences_words_syllables():
    strings_and_words = [
        ("já", (1, 1, 1)),
        ("já já", (1, 2, 2)),
        ("já     já\tjá", (1, 3, 3)),
        ("Þetta. Eru. Fjórar. Setningar", (4, 4, 9)),
    ]
    for string, (num_sents, num_words, num_syllables) in strings_and_words:
        assert get_sentence_word_syllable_counts(string) == (
            num_sents,
            num_words,
            num_syllables,
        ), f"Expected {num_syllables} syllables, {num_words} words and {num_sents} sentences in {string}"


def test_flesch_score_range():
    for text in text_dict:
        assert readability.FleschKincaidScorer.get_score_from_text(text) > 0
        assert readability.FleschKincaidScorer.get_score_from_text(text) < 100

    assert readability.FleschKincaidScorer.get_score_from_text(
        easy_text1
    ) > readability.FleschKincaidScorer.get_score_from_text(
        medium_text1
    ), "Expected easy text to be easier than medium text"
    assert readability.FleschKincaidScorer.get_score_from_text(
        medium_text1
    ) > readability.FleschKincaidScorer.get_score_from_text(
        medium_hard_text
    ), "Expected medium text to be easier than medium hard text"
    assert readability.FleschKincaidScorer.get_score_from_text(
        medium_text2
    ) > readability.FleschKincaidScorer.get_score_from_text(
        medium_hard_text
    ), "Expected medium text to be easier than medium hard text"
    assert readability.FleschKincaidScorer.get_score_from_text(
        medium_hard_text
    ) > readability.FleschKincaidScorer.get_score_from_text(
        hard_text1
    ), "Expected medium hard text to be easier than hard text"


def test_flesch_score_against_estimate():
    for text in text_dict:
        flesch_score = readability.FleschKincaidScorer.get_score_from_text(text)
        feedback = readability.FleschKincaidScorer.get_feedback(flesch_score)

        assert (
            flesch_score > text_dict[text][0]
        ), f"Expected text to be easier than estimate: {text_dict[text][0]}, was {flesch_score}. Feedback: {feedback}. First sentence: {text.split('.')[0]}"

        assert (
            flesch_score < text_dict[text][1]
        ), f"Expected text to be harder than estimate: {text_dict[text][1]}, was {flesch_score}. Feedback: {feedback}. First sentence: {text.split('.')[0]}"

    for text in not_real_texts:
        flesch_score = readability.FleschKincaidScorer.get_score_from_text(text)
        feedback = readability.FleschKincaidScorer.get_feedback(flesch_score)

        assert (
            flesch_score > not_real_texts[text][0]
        ), f"Expected text to be easier than estimate: {not_real_texts[text][0]}, was {flesch_score}. Feedback: {feedback}. Feedback: {feedback}. First sentence: {text.split('.')[0]}"

        assert (
            flesch_score < not_real_texts[text][1]
        ), f"Expected text to be harder than estimate: {not_real_texts[text][1]}, was {flesch_score}. Feedback: {feedback}. Feedback: {feedback}. First sentence: {text.split('.')[0]}"



#!/usr/bin/env python

"""
Easy way to look at input and output together.
To run for the file 'prufa.txt':
$ python diffchecker.py prufa.txt

"""
from typing import Dict, Iterable, Iterator, Set, Union

import argparse
import sys

from reynir_correct.wrappers import check_errors

# File types for UTF-8 encoded text files
ReadFile = argparse.FileType("r", encoding="utf-8")
WriteFile = argparse.FileType("w", encoding="utf-8")

# Define the command line arguments
parser = argparse.ArgumentParser(description="Corrects Icelandic text")

parser.add_argument(
    "infile",
    nargs="?",
    type=ReadFile,
    default=sys.stdin,
    help="UTF-8 text file to correct",
)


def gen(f: Iterator[str]) -> Iterable[str]:
    """Generate the lines of text in the input file"""
    yield from f


def main() -> None:
    options: Dict[str, Union[str, bool, Set[str]]] = {}
    options["format"] = "text"  # text, json, csv, m2
    options["annotations"] = True
    options["all_errors"] = True
    # options["input"] = open("prufa.txt", "r")
    options["one_sent"] = False
    options["generate_suggestion_list"] = True
    options["ignore_comments"] = True  # Only used here
    options["annotate_unparsed_sentences"] = True
    options["suppress_suggestions"] = False
    options["replace_html_escapes"] = True
    options["ignore_wordlist"] = set()
    options["spaced"] = False
    options["print_all"] = True
    options["ignore_rules"] = set()
    options["suggest_not_correct"] = False
    options["extra_config"] = "config/ExtraWords.conf"
    options["extra_tone_of_voice"] = True

    args = parser.parse_args()
    infile = args.infile
    if infile is sys.stdin and sys.stdin.isatty():
        # terminal input is empty, most likely no value was given for infile:
        # Nothing we can do
        print("No input has been given, nothing can be returned")
        sys.exit(1)
    itering = gen(infile)
    for sent in itering:
        sent = sent.strip()
        if not sent:
            continue
        if sent.startswith("#"):
            # Comment string, want to show it with the examples
            if not options.get("ignore_comments"):
                print(sent)
            continue
        options["input"] = sent
        x = check_errors(**options)
        # Here we can compare x to gold by zipping sentences
        # from output and gold together and iterating in a for loop
        print(sent.strip())
        if x:
            print(x)
        print("=================================")
    """
    # Used when output shouldn't be split into sentences
    options["input"] = infile
    x = check_errors(**options)
    if x:
        print(x)
    print("=================================")
    """


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Easy way to compare error categories in Ritmyndir and the metadata,
checking for new ones that need to be added.

$ python ritmyndirchecker.py

"""
from reynir_correct.settings import RitmyndirDetails
from reynir_correct.settings import Ritmyndir

from typing import (
    Set,
)

# Error codes that are not explicitly an error
IGNORE = set(["GAM", "SO-ÞGF4ÞF", "OSB-BMYND", "SJALD", "STAD", "AV"])


def main() -> None:
    ritmyndir = Ritmyndir()
    ritmyndir_details = RitmyndirDetails()
    allcodes: Set[str] = set()
    for entry in ritmyndir.DICT:
        allcodes.add(ritmyndir.get_code(entry))
    detcodes: Set[str] = set()
    for keycode in ritmyndir_details.DICT:
        detcodes.add(keycode)

    # Compare
    newcodes = allcodes - detcodes - IGNORE
    outdated = detcodes - allcodes - IGNORE

    print("=====================")
    print("New codes:")
    for item in newcodes:
        print("\t{}".format(item))
    print("=====================")
    print("Outdated/unused codes:")
    for item in outdated:
        print("\t{}".format(item))


if __name__ == "__main__":
    main()



# type: ignore
"""

    test_annotator.py

    Tests for GreynirCorrect module

    Copyright © 2025 by Miðeind ehf.

    This software is licensed under the MIT License:

        Permission is hereby granted, free of charge, to any person
        obtaining a copy of this software and associated documentation
        files (the "Software"), to deal in the Software without restriction,
        including without limitation the rights to use, copy, modify, merge,
        publish, distribute, sublicense, and/or sell copies of the Software,
        and to permit persons to whom the Software is furnished to do so,
        subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


    This module tests the sentence-level annotation functionality
    of GreynirCorrect.

    Potential test sentences:

        Tillaga formanna þingflokkana var rædd í gær, eftir að frumvarpið var sett í kælir.
        Manninum á verkstæðinu vantaði hamar.
        Það var auðséð að henni langaði að fara til sólarlanda.
        Mitt í hamaganginum hlakkaði Jónasi til að fá sér kakó.
        Jón hefur aðra sögu að segja heldur en Páll.
        Ég hætti við að fara víst að Sigga var veik.
        Víst að Sigga var veik hætti ég við að fara.
        Frá því ég sá hana fyrst var ég ástfanginn.
        Annað hvort er þetta della eða þetta virkar vel. Annaðhvort systkinanna mun örugglega greiða mér.
        Fjöldi Evrópuríkja hafa mótmælt áformum Breta. Ég var viss um að fjöldi stuðningsmanna Liverpool myndu fagna.
        Einn af drengjunum voru komnir með flensu, meðan einn af læknunum þurftu að fara heim.
        Allt Viðreisnarfólk, sem og Píratar, tóku þátt í atkvæðagreiðslunni.
        Ég gekk frá skrifborðinu, áður en að ég ók bílnum heim. Ég hef verið frískur síðan að ég tók fúkkalyfið.
        Meðan að tölvan er í viðgerð, get ég lítið unnið.
        Mér var ekki sama um þetta, jafnvel þó hjúkrunarfræðingurinn reyndi að hughreysta mig.
        Þó veðrið væri vont, gátum við þvegið bílinn.
        Hundurinn hans Páls fóru í bað í gær.
        Allir kettirnir í götunni var að elta mýs.
        Samhliða leiksýningin talaði ég við Páll um vandamálið.
        Ég las síðustu bók Guðrúnar (sú sem ég minntist á við þig) og fannst hún býsna góð.

"""


import pytest

import reynir_correct

from .utils import check_sentence


@pytest.fixture(scope="module")
def api() -> reynir_correct.GreynirCorrectAPI:
    """Provide a module-scoped GreynirCorrect instance as a test fixture"""
    r = reynir_correct.GreynirCorrectAPI.from_options()
    yield r


def dump(tokens):
    print("\n{0} tokens:\n".format(len(tokens)))
    for token in tokens:
        print("{0}".format(token))
        err = token.error_description
        if err:
            print("   {0}: {1}".format(token.error_code, err))


def test_multiword_phrases(api):
    s = "Einn af drengjunum fór í sund af gefnu tilefni."
    check_sentence(api, s, [(6, 8, "P_afað")])


def test_error_finder(api):
    """Test errors that are found by traversing the detailed
    parse tree in checker.py (ErrorFinder class)"""
    s = "Einn af drengjunum fóru í sund."
    check_sentence(api, s, [(3, 3, "P_NT_EinnAf")])
    s = "Fjöldi þingmanna greiddu atkvæði gegn tillögunni."
    check_sentence(api, s, [(2, 2, "P_NT_FjöldiHluti")])
    s = "Jón borðaði ís þar sem að hann var svangur."
    check_sentence(api, s, [(5, 5, "P_NT_Að/w")])
    s = 'Jón "borðaði" ís þar sem að hann var svangur.'
    check_sentence(api, s, [(1, 1, "N001"), (3, 3, "N001"), (7, 7, "P_NT_Að/w")])
    s = "Jón borðaði ís þó hann væri svangur."
    check_sentence(api, s, [(3, 3, "P_NT_ÞóAð")])
    s = 'Jón "borðaði" ís þó hann væri svangur.'
    check_sentence(api, s, [(1, 1, "N001"), (3, 3, "N001"), (5, 5, "P_NT_ÞóAð")])
    s = "Jón borðaði ís jafnvel þó hann væri svangur."
    check_sentence(api, s, [(3, 4, "P_NT_ÞóAð")])
    s = 'Jón "borðaði" ís jafnvel þó hann væri svangur.'
    check_sentence(api, s, [(1, 1, "N001"), (3, 3, "N001"), (5, 6, "P_NT_ÞóAð")])
    s = "Jón borðaði ís þótt hann væri svangur."
    check_sentence(api, s, [])
    s = 'Jón "borðaði" ís þótt hann væri svangur.'
    check_sentence(api, s, [(1, 1, "N001"), (3, 3, "N001")])
    s = "Ég féll fyrir annað hvort fegurð hennar eða gáfum."
    check_sentence(api, s, [(3, 4, "P_NT_AnnaðHvort")])
    s = "Ég talaði við annaðhvort barnanna."
    check_sentence(api, s, [(3, 3, "P_NT_Annaðhvort")])
    s = "Ég hef verið slappur frá því ég fékk sprautuna."
    check_sentence(api, s, [(5, 5, "P_NT_FráÞvíAð")])
    s = "Ég hef verið slappur allt frá því ég fékk sprautuna."
    check_sentence(api, s, [(6, 6, "P_NT_FráÞvíAð")])
    s = "Friðgeir vildi vera heima víst að Sigga yrði að vera heima."
    # TODO no longer annotated, due to changes in verb frames
    # check_sentence(rc, s, [(4, 4, "P_NT_VístAð")])
    s = "Víst að Sigga var heima ákvað Friðgeir að vera heima."
    check_sentence(api, s, [(0, 0, "P_NT_VístAð")])
    s = "Friðgeir taldi víst að Sigga yrði að vera heima."
    check_sentence(api, s, [])
    s = "Ég er ekki meiri fáviti heldur en þú."
    check_sentence(api, s, [(4, 4, "T001/w"), (5, 5, "P_NT_Heldur/w")])


def test_ordinals(api):
    # NOTE: Commented out as this functionality increases the number of
    # false positives on the iceErrorCorpus test set.
    # s = "4. barnið fæddist í gær, en það er 3. strákur þeirra hjóna."
    # check_sentence(rc, s, [(0, 0, "X_number4word"), (8, 8, "X_number4word")])
    # sent = rc.parse_single(s)
    # assert sent.annotations[0].suggest == "Fjórða"
    # assert sent.annotations[1].suggest == "þriðji"
    s = "5. Ákæran beinist gegn Jóni og Friðberti."
    check_sentence(api, s, [])
    # s = "2. deildin fer vel af stað í vetur."
    # check_sentence(rc, s, [(0, 0, "X_number4word")])
    s = "XVII. kafli: Um landsins gagn og nauðsynjar."
    check_sentence(api, s, [])


def test_pronoun_annara(api):
    s = (
        "Allir í hans bekk, auk nokkurra nemenda úr öðrum bekkjum, "
        "umsjónakennara og fjögurra annara kennara "
        "hafa verið sendir í sjö daga sóttkví."
    )
    check_sentence(api, s, [(12, 12, "S004"), (15, 15, "R4RR")])
    s = " Mér er annara um símann minn en orðspor mitt."
    # TODO 'annara' is changed to 'annarra' due to better token-level data
    # and a difficult syntax pattern
    # check_sentence(rc, s, [])


def test_impersonal_verbs(api):
    s = "Mig hlakkaði til."
    check_sentence(api, s, [(0, 0, "P_WRONG_CASE_þf_nf")])
    s = "Mér hlakkaði til."
    check_sentence(api, s, [(0, 0, "P_WRONG_CASE_þgf_nf")])
    s = "Ég dreymdi köttinn."
    check_sentence(api, s, [(0, 0, "P_WRONG_CASE_nf_þf")])
    s = "Mér dreymdi köttinn."
    check_sentence(api, s, [(0, 0, "P_WRONG_CASE_þgf_þf")])
    # The following should not parse
    s = "Ég dreymdi kettinum."
    check_sentence(api, s, None)
    s = "Páli, sem hefur verið landsliðsmaður í fótbolta í sjö ár, " "langaði að horfa á sjónvarpið."
    check_sentence(api, s, [(0, 11, "P_WRONG_CASE_þgf_þf")])
    s = "Pál, sem hefur verið landsliðsmaður í fótbolta í sjö ár, " "langaði að horfa á sjónvarpið."
    check_sentence(api, s, [])
    s = "Pál kveið fyrir skóladeginum."
    check_sentence(api, s, [(0, 0, "P_WRONG_CASE_þf_nf")])
    s = "Páli kveið fyrir skóladeginum."
    check_sentence(api, s, [(0, 0, "P_WRONG_CASE_þgf_nf")])
    s = "Unga fólkinu skortir aðhald."
    check_sentence(api, s, [(0, 1, "P_WRONG_CASE_þgf_þf")])
    # FIXME:
    # s = "Ég held að músinni hafi kviðið fyrir að hitta köttinn."
    # check_sentence(rc, s, [(3, 3, "P_WRONG_CASE_þgf_nf")])
    s = "Hestinum Grímni vantaði hamar."
    # s = "Hestinum Skjóna vantaði hamar."
    check_sentence(api, s, [(0, 1, "P_WRONG_CASE_þgf_þf")])
    s = "Stóra manninum sem vinnur á verkstæðinu vantaði hamar."
    check_sentence(api, s, [(0, 5, "P_WRONG_CASE_þgf_þf")])


def test_foreign_sentences(api):
    check_sentence(
        api,
        "It was the best of times, it was the worst of times.",
        [(0, 13, "E004")],
        is_foreign=True,
    )
    check_sentence(
        api,
        "Praise the Lord.",
        [(0, 1, "E004")],  # Note: the tokenizer amalgams 'Praise the Lord' into one token
        is_foreign=True,
    )
    check_sentence(
        api,
        "Borðaðu Magnyl og Xanax eagerly in Rushmore for the holidays.",
        [(0, 8, "E004")],
        is_foreign=True,
    )  # Note: Example needed to be made longer due to 'in' appearing as an Icelandic error


def test_number(api):
    check_sentence(api, "Vinnuvika sjómanna eru 7 heilir dagar.", [(2, 2, "P_NT_ÍTölu")])
    check_sentence(api, "Hjón borðar matinn sinn.", [(1, 1, "P_NT_ÍTölu")])
    check_sentence(api, "Ég borðum matinn minn.", [(1, 1, "P_NT_ÍTölu")])


def test_correct_sentences(api):
    check_sentence(api, "Pál langaði að horfa á sjónvarpið.", [])
    check_sentence(api, "Mig dreymdi mús sem elti kött.", [])
    check_sentence(
        api,
        "Ég held að músin hafi kviðið fyrir að hitta köttinn.",
        [],
        ignore_warnings=True,
    )
    check_sentence(api, "Músin kveið fyrir að hitta köttinn.", [])
    check_sentence(
        api,
        "Páll hlakkaði til jólanna og að hitta strákinn sem hlakkaði til páskanna.",
        [],
    )
    check_sentence(api, "Ég hlakka til að sjá nýju Aliens-myndina.", [])


def test_corrected_meanings(api) -> None:
    s = """
    Þannig fundust stundum engin bréfaskipti á milli lífsförunauta í annars ríkulegum bréfasöfnum.
    """
    check_sentence(api, s.rstrip(), [])
    s = """
    Þeir hafa líka þennan Beach Boys-hljóm og virkilega fallegar raddanir,"
    sagði Jardine, en platan hans nefnist A Postcard fram California.
    """
    # Note: "A Postcard" is tokenized as one entity token and should not
    # be reported as an error or annotation
    check_sentence(
        api,
        s.rstrip(),
        [(11, 11, "N001")],
    )


def test_lhþt_variant(api) -> None:
    """Check for a regression in the handling of LHÞT variants in BinPackage"""
    s = (
        "Minna þekkt eru sem dæmi rafgas og kvarka-límeindarafgas, "
        "Bose-Einstein þétting og oddskiptaeindaþétting, sérstætt efni, "
        "vökvakristall, ofurstraumefni og ofurþéttefni og einnig "
        "meðseglunar- og járnseglunarhamir segulmagnaðra efna."
    )
    try:
        api.correct(s)
    except ValueError:
        assert False, "Regression in handling of LHÞT variants in BinPackage"


if __name__ == "__main__":
    from reynir_correct import GreynirCorrect

    gc = GreynirCorrect()
    test_multiword_phrases(gc)
    test_impersonal_verbs(gc)
    test_error_finder(gc)
    test_correct_sentences(gc)
    test_foreign_sentences(gc)
    test_number(gc)
    test_corrected_meanings(gc)
